@@ -1,5 +1,5 @@
 """Top-level package for choixe."""
 
 __author__ = """daniele de gregorio"""
-__email__ = 'daniele.degregorio@eyecan.ai'
-__version__ = '0.1.7'
+__email__ = "daniele.degregorio@eyecan.ai"
+__version__ = "0.1.11"
