@@ -1,7 +1,11 @@
 from enum import Enum, auto
 from typing import Union
-from choixe.directives import Directive, DirectiveConsumer, DirectiveFactory
-from pathlib import Path
+from choixe.directives import (
+    Directive,
+    DirectiveAT,
+    DirectiveConsumer,
+    DirectiveFactory,
+)
 
 
 class ImporterType(Enum):
@@ -13,19 +17,11 @@ class ImporterType(Enum):
         return list([c.name.lower() for c in cls])
 
     @classmethod
-    def get_type(cls, value: str) -> Union['ImporterType', None]:
+    def get_type(cls, value: str) -> Union["ImporterType", None]:
         return ImporterType[value.upper()]
 
-    # @classmethod
-    # def get_type_as_string(cls, value: str) -> str:
-    #     tp = cls.get_type(value)
-    #     if isinstance(tp, ImporterType):
-    #         return tp.name
-    #     return 'None'
-
 
 class Importer(DirectiveConsumer):
-
     def __init__(self, directive: Directive):
         super().__init__(directive=directive)
 
@@ -39,7 +35,7 @@ class Importer(DirectiveConsumer):
         if self.is_valid():
             if len(self._directive.args) > 0:
                 return self._directive.args[0]
-        return ''
+        return ""
 
     @property
     def type(self):
@@ -48,8 +44,26 @@ class Importer(DirectiveConsumer):
         return None
 
     @classmethod
-    def from_string(cls, value: str) -> 'Importer':
+    def from_string(cls, value: str) -> "Importer":
         directive = DirectiveFactory.build_directive_from_string(value)
         if directive:
             return Importer(directive=directive)
         return None
+
+    @property
+    def options(self):
+        if self.is_valid():
+            return self._directive.args[1:]
+        return []
+
+    @property
+    def default_value(self):
+        if self.is_valid():
+            return self._directive.default_value
+        return None
+
+    @classmethod
+    def generate_importer_directive(cls, importer_type: ImporterType, path: str):
+        return DirectiveAT.generate_directive_string(
+            str(importer_type.name).lower(), [path]
+        )
