@@ -7,13 +7,13 @@ def test_check(sample_configurations_data):
     runner = CliRunner()
 
     for cfg_data in sample_configurations_data:
-        filename = cfg_data['filename']
-        has_placeholders = cfg_data['has_placeholders']
+        filename = cfg_data["filename"]
+        has_placeholders = cfg_data["has_placeholders"]
 
-        result = runner.invoke(check, ['-c', filename])
+        result = runner.invoke(check, ["-c", filename])
         assert result.exit_code == (1 if has_placeholders else 0)
 
-        result = runner.invoke(check, ['-c', filename, '--noclose'])
+        result = runner.invoke(check, ["-c", filename, "--noclose"])
         assert result.exit_code == 0
 
 
@@ -21,8 +21,9 @@ def test_check_wrong(sample_configurations_data):
     runner = CliRunner()
 
     for cfg_data in sample_configurations_data:
-        filename = Path(str(cfg_data['filename']) + "@IMPOSSIBLE_ST3ING!")
-        has_placeholders = cfg_data['has_placeholders']
+        filename = Path(str(cfg_data["filename"]) + "@IMPOSSIBLE_ST3ING!")
+        has_placeholders = cfg_data["has_placeholders"]
+        print(has_placeholders)
 
-        result = runner.invoke(check, ['-c', filename])
+        result = runner.invoke(check, ["-c", filename])
         assert result.exit_code == 1
